/*
 * GnuAPDF Library
 *   Jon Sevy, jsevy@cs.drexel.edu
 * 
 * Based on GnuJPDF Library, authored by (among others)
 *   Peter T Mount, http://www.retep.org.uk/pdf/
 *   Eric Z. Beard, ericzbeard@hotmail.com
 *   Gilbert DeLeeuw, gil1@users.sourceforge.net
 *   Matthew Hreljac, mhreljac@hotmail.com  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package gnu.apdf;


import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.Vector;

import android.graphics.Typeface;


/**
 * 
 * This class is the base of the PDF library. A PDFDocument class is created for a document, and each page, object,
 * annotation, etc is added to the document. The content of each page is drawn using the class PDFCanvas. Once complete,
 * the document can be written to an OutputStream, which will generate well-formed PDF that will reproduce the text and
 * images drawn on the pages.
 * 
 * <p>
 * The typical flow for use of the PDF library is as follows: <code><pre>
 * // Create a PDF document, a page in the document, and get that page's PDFCanvas 
 * PDFDocument pdfDoc = new PDFDocument(); 
 * PDFPage page = new PDFPage(PageFormat.LETTER); 
 * pdfDoc.add(page); 
 * PDFCanvas pdfCanvas = page.getCanvas();
 * 
 * // do drawing commands on the PDFCanvas associated with the page as you would on any other Canvas 
 * myDrawDesign(pdfCanvas);
 * 
 * // add and draw on other pages as desired
 * 
 * // write the PDF content for the page to a file 
 * BufferedOutputStream fileStream = new BufferedOutputStream(new FileOutputStream(file)); 
 * pdfDoc.write(outStream); 
 * fileStream.flush();
 * fileStream.close();
 * </pre></code>
 * </p>
 * 
 * @author Peter T Mount, http://www.retep.org.uk/pdf/
 * @author Eric Z. Beard, ericzbeard@hotmail.com
 * @author Gilbert DeLeeuw, gil1@users.sourceforge.net
 * @author Jon Sevy, jsevy@cs.drexel.edu
 */
public class PDFDocument implements Serializable
{
    
    /*
     * NOTE: This class originated in uk.org.retep.pdf, by Peter T. Mount, and it has been modified by Eric Z. Beard,
     * ericzbeard@hotmail.com. The package name was changed to gnu.jpdf and several inner classes were moved out into
     * their own files.
     */
    
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    /**
     * This is used to allocate objects a unique serial number in the document.
     */
    protected int objser;
    
    /**
     * This vector contains each indirect object within the document.
     */
    protected Vector<PDFObject> objects;
    
    /**
     * This is the Catalog object, which is required by each PDF Document
     */
    private PDFCatalog catalog;
    
    /**
     * This is the info object. Although this is an optional object, we include it.
     */
    private PDFInfo info;
    
    /**
     * This is the Pages object, which is required by each PDF Document
     */
    private PDFPageList pdfPageList;
    
    /**
     * This is the Outline object, which is optional
     */
    private PDFOutline outline;
    
    /**
     * This holds a PDFObject describing the default border for annotations. It's only used when the document is being
     * written.
     */
    protected PDFObject defaultOutlineBorder;
    
    /**
     * <p>
     * This page mode indicates that the document should be opened just with the page visible. This is the default
     * </p>
     */
    public static final int USENONE = 0;
    
    /**
     * <p>
     * This page mode indicates that the Outlines should also be displayed when the document is opened.
     * </p>
     */
    public static final int USEOUTLINES = 1;
    
    /**
     * <p>
     * This page mode indicates that the Thumbnails should be visible when the document first opens.
     * </p>
     */
    public static final int USETHUMBS = 2;
    
    /**
     * <p>
     * This page mode indicates that when the document is opened, it is displayed in full-screen-mode. There is no menu
     * bar, window controls nor any other window present.
     * </p>
     */
    public static final int FULLSCREEN = 3;
    
    /**
     * <p>
     * These map the page modes just defined to the pagemodes setting of PDF.
     * </p>
     */
    public static final String PDF_PAGE_MODES[] = { "/UseNone", "/UseOutlines", "/UseThumbs", "/FullScreen" };
    
    /**
     * These are used to provide unique names for named resources - appended to the type string to generate a uniue
     * identifier.
     */
    private int objectID = 0;
    
    /**
     * This holds the current fonts
     */
    private Vector<PDFFont> fonts;
    
    
    
    /**
     * <p>
     * This creates a PDF document with the default pagemode
     * </p>
     */
    public PDFDocument()
    {
        this(USENONE);
    }
    
    
    
    /**
     * <p>
     * This creates a PDF document
     * </p>
     * 
     * @param pagemode
     *            an int, determines how the document will present itself to the viewer when it first opens.
     */
    public PDFDocument(int pagemode)
    {
        objser = 1;
        objects = new Vector<PDFObject>();
        fonts = new Vector<PDFFont>();
        
        // Now create some standard objects
        add(pdfPageList = new PDFPageList());
        add(catalog = new PDFCatalog(pdfPageList, pagemode));
        add(info = new PDFInfo());
        
        // Acroread on linux seems to die if there is no root outline
        add(getOutline());
    }
    
    
    
    /**
     * This adds a top level object to the document.
     * 
     * <p>
     * Once added, it is allocated a unique serial number, as well as a unique name relevant to the object type (e.g.,
     * /Image47 or /Pattern19)
     * </p>
     * <p>
     * <b>Note:</b> Not all objects are added directly using this method. Some objects which have Kids (in PDF
     * sub-objects or children are called Kids) will have their own add() method, which will call this one internally.
     * 
     * @param obj
     *            The PDFObject to add to the document
     * @return the unique serial number for this object.
     */
    public synchronized int add(PDFObject obj)
    {
        objects.addElement(obj);
        obj.objser = objser++; // create a new serial number
        obj.pdfDocument = this; // so they can find the document they belong to
        
        // If its a page, then add it to the pages collection
        if (obj instanceof PDFPage)
            pdfPageList.add((PDFPage) obj);
        
        // set a unique name for the object
        obj.setName(getUniqueObjectName(obj));
        
        return obj.objser;
    }
    
    
    
    /**
     * <p>
     * This returns a specific page
     * </p>
     * 
     * @param page
     *            page number to return
     * @return PDFPage at that position
     */
    public PDFPage getPage(int page)
    {
        return pdfPageList.getPage(page);
    }
    
    
    
    /**
     * @return the root outline
     */
    public PDFOutline getOutline()
    {
        if (outline == null)
        {
            outline = new PDFOutline();
            catalog.setOutline(outline);
        }
        return outline;
    }
    
    
    
    /**
     * This returns a font of the specified type and typeface. If the font has not been defined, it creates a new font
     * in the PDF document, and returns it.
     * 
     * @param type
     *            PDF Font Type - usually "/Type1"
     * @param typeface
     *            Android typeface
     * @return PDFFont defining this font
     */
    public PDFFont getFont(String type, Typeface typeface)
    {
        // Search the fonts on this page, and return one that matches this
        // Typeface.
        // This keeps the number of font definitions down to one per font/style
        // Use a temp version of the PDF font to see if it's already present
        PDFFont tempFont = new PDFFont("temp", type, typeface);
        for (PDFFont ft : fonts)
        {
            if (ft.equals(tempFont))
                return ft;
        }
        
        // the font wasn't found, so add it
        objectID++;
        PDFFont ft = new PDFFont("/F" + objectID, type, typeface);
        add(ft);
        fonts.addElement(ft);
        return ft;
    }
    
    
    
    /**
     * Generate a unique name for the object, using the object type name as base and a running count of all of the
     * objects added to generate a numerical suffix for uniqueness
     * 
     * @param object
     * @return Uniques name for the object
     */
    private String getUniqueObjectName(PDFObject object)
    {
        objectID++;
        object.setName(object.getType() + objectID);
        return object.getName();
    }
    
    
    
    /**
     * <p>
     * Set the PDFInfo object, which contains author, title, keywords, etc
     * </p>
     * 
     * @param info
     *            a PDFInof object
     */
    public void setPDFInfo(PDFInfo info)
    {
        this.info = info;
    }
    
    
    
    /**
     * <p>
     * Get the PDFInfo object, which contains author, title, keywords, etc
     * </p>
     * 
     * @return the PDFInfo object for this document.
     */
    public PDFInfo getPDFInfo()
    {
        return this.info;
    }
    
    
    
    /**
     * This writes the document to an OutputStream.
     * 
     * <p>
     * <b>Note:</b> You can call this as many times as you wish, as long as the calls are not running at the same time.
     * 
     * <p>
     * Also, objects can be added or amended between these calls.
     * 
     * <p>
     * Also, the OutputStream is not closed, but will be flushed on completion. It is up to the caller to close the
     * stream.
     * 
     * @param os
     *            OutputStream to write the document to
     * @exception IOException
     *                on error
     */
    public void write(OutputStream os) throws IOException
    {
        PDFOutput pos = new PDFOutput(os);
        
        // Write each object to the OutputStream. We call via the output
        // as that builds the xref table
        for (PDFObject o : objects)
        {
            pos.write(o);
        }
        
        // Finally close the output, which writes the xref table.
        pos.close();
        
        // and flush the output stream to ensure everything is written.
        os.flush();
    }
    
} // end class PDFDocument
