/*
 * GnuAPDF Library
 *   Jon Sevy, jsevy@cs.drexel.edu
 * 
 * Based on GnuJPDF Library, authored by (among others)
 *   Peter T Mount, http://www.retep.org.uk/pdf/
 *   Eric Z. Beard, ericzbeard@hotmail.com
 *   Gilbert DeLeeuw, gil1@users.sourceforge.net
 *   Matthew Hreljac, mhreljac@hotmail.com  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package gnu.apdf;


import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.HashMap;

import android.graphics.Typeface;


/**
 * This class defines a font within a PDF document.
 * 
 * @author Peter T Mount,http://www.retep.org.uk/pdf/
 * @author Eric Z. Beard, ericzbeard@hotmail.com
 * @author Gilbert DeLeeuw, gil1@users.sourceforge.net
 * @author Jon Sevy, jsevy@cs.drexel.edu
 */
public class PDFFont extends PDFObject implements Serializable
{
    
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    /*
     * NOTE: The original class is the work of Peter T. Mount, who released it in the uk.org.retep.pdf package. It was
     * modified by Eric Z. Beard as follows: The package name was changed to gnu.pdf. The formatting was changed a
     * little bit It is still licensed under the LGPL.
     */
    
    /**
     * The name of the font used for reference in the document
     */
    private String name;
    
    /**
     * The PDF type of the font, usually /Type1
     */
    private String type;
    
    /**
     * The font's PDF name, including the slash
     */
    private String pdfName;
    
    /**
     * The PDF Style, ie: BOLD, ITALIC, etc
     */
    private int style;
    
    
    
    /**
     * This constructs a default PDFFont based on the default Android typeface.
     */
    protected PDFFont()
    {
        this("/F1", "/Type1", Typeface.DEFAULT);
    }
    
    
    
    /**
     * Constructs a PDFFont. This will attempt to map the font from a known Java font name to that in PDF, defaulting to
     * Helvetica if not possible.
     * 
     * @param name
     *            The name for the font to be used in the document, ie /F1
     * @param type
     *            The pdf type, ie /Type1
     * @param pdfName
     *            The font name, ie Helvetica
     * @param style
     *            The android.graphics.Typeface style, e.g., Typeface.NORMAL
     */
    public PDFFont(String name, String type, Typeface typeface)
    {
        super("/Font");
        this.name = name;
        this.type = type;
        
        // luckily the Android styles align with the PDF styles:
        // NORMAL = 0, BOLD = 1, ITALIC = 2, BOLD_ITALIC = 3
        this.style = typeface.getStyle();
        
        // attempt to translate the typeface from Android to PDF
        if (base14.containsKey(typeface))
        {
            String[] fontFamily = base14.get(typeface);
            this.pdfName = fontFamily[style];
            // System.out.println("Setting a font style to: " + this.font);
        }
        else
        {
            // use default PDF Font
            this.pdfName = "/Helvetica";
            
        }
    }
    
    
    
    /**
     * This is the most common method to use.
     * 
     * @return the Font name within the PDF document.
     */
    public String getName()
    {
        return name;
    }
    
    
    
    /**
     * @return the Font's PDF type
     */
    public String getType()
    {
        return type;
    }
    
    
    
    /**
     * @return The PDF Font name
     */
    public String getFont()
    {
        return pdfName;
    }
    
    
    
    /**
     * @return the font style.
     * @see java.awt.Font
     */
    public int getStyle()
    {
        return style;
    }
    
    
    
    /**
     * @param os
     *            OutputStream to send the object to
     * @exception IOException
     *                on error
     */
    public void write(OutputStream os) throws IOException
    {
        // Write the object header
        writeStart(os);
        
        // now the objects body
        os.write("/Subtype ".getBytes());
        os.write(type.getBytes());
        os.write("\n/Name ".getBytes());
        os.write(name.getBytes());
        os.write("\n/BaseFont ".getBytes());
        os.write(pdfName.getBytes());
        // The performance problem in Bug#106693 comments out the
        // encoding line, and removes the /WinAnsiEncoding. I'm going
        // to leave them in, as the Encoding fixes another problem.
        os.write("\n/Encoding ".getBytes());
        os.write("/WinAnsiEncoding".getBytes());
        // os.write(encoding.getBytes());
        os.write("\n".getBytes());
        
        // finish off with its footer
        writeEnd(os);
    }
    
    
    
    /**
     * Method used to compare Fonts
     * 
     * @param type
     *            The pdf type, ie /Type1
     * @param pdfName
     *            The font name, ie Helvetica
     * @return True if types and pdf names are equal
     */
    protected boolean equals(String type, String pdfName)
    {
        return (this.type.equals(type) && this.pdfName.equalsIgnoreCase(pdfName));
    }
    
    /**
     * This maps from the standard Android Typefaces to the standard PDF fonts
     */
    private static HashMap<Typeface, String[]> base14 = new HashMap<Typeface, String[]>()
    {
        /**
         * 
         */
        private static final long serialVersionUID = 1L;
        
        {
            // Android typeface
            // NORMAL
            // BOLD
            // ITALIC
            // BOLD+ITALIC
            put(Typeface.DEFAULT, new String[] { "/Helvetica", "/Helvetica-Bold", "/Helvetica-Oblique", "/Helvetica-BoldOblique" });
            put(Typeface.MONOSPACE, new String[] { "/Courier", "/Courier-Bold", "/Courier-Oblique", "/Courier-BoldOblique" });
            put(Typeface.SANS_SERIF, new String[] { "/Helvetica", "/Helvetica-Bold", "/Helvetica-Oblique", "/Helvetica-BoldOblique" });
            put(Typeface.SERIF, new String[] { "/Times-Roman", "/Times-Bold", "/Times-Italic", "/Times-BoldItalic" });
            put(Typeface.DEFAULT_BOLD, new String[] { "/Helvetica", "/Helvetica-Bold", "/Helvetica-Oblique", "/Helvetica-BoldOblique" });
        }
    };
    
}
