/*
 * GnuAPDF Library
 *   Jon Sevy, jsevy@cs.drexel.edu
 * 
 * Based on GnuJPDF Library, authored by (among others)
 *   Peter T Mount, http://www.retep.org.uk/pdf/
 *   Eric Z. Beard, ericzbeard@hotmail.com
 *   Gilbert DeLeeuw, gil1@users.sourceforge.net
 *   Matthew Hreljac, mhreljac@hotmail.com  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package gnu.apdf;


import java.io.Serializable;
import java.util.Arrays;
import java.util.Vector;


/**
 * Class representing the format of a page, including constant definitions for many common page sizes.
 * 
 * @author jsevy
 *
 */
public class PDFPageFormat implements Serializable
{
    
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    /**
     * Constants defining parameters for the most common page sizes
     */
    public static final String A0_NAME = "A0";
    public static final int A0_WIDTH = 2384;
    public static final int A0_HEIGHT = 3371;
    
    public static final String A1_NAME = "A1";
    public static final int A1_WIDTH = 1685;
    public static final int A1_HEIGHT = 2384;
    
    public static final String A2_NAME = "A2";
    public static final int A2_WIDTH = 1190;
    public static final int A2_HEIGHT = 1684;
    
    public static final String A3_NAME = "A3";
    public static final int A3_WIDTH = 842;
    public static final int A3_HEIGHT = 1190;
    
    public static final String A4_NAME = "A4";
    public static final int A4_WIDTH = 595;
    public static final int A4_HEIGHT = 842;
    
    public static final String A5_NAME = "A5";
    public static final int A5_WIDTH = 420;
    public static final int A5_HEIGHT = 595;
    
    public static final String A6_NAME = "A6";
    public static final int A6_WIDTH = 298;
    public static final int A6_HEIGHT = 420;
    
    public static final String A7_NAME = "A7";
    public static final int A7_WIDTH = 210;
    public static final int A7_HEIGHT = 298;
    
    public static final String A8_NAME = "A8";
    public static final int A8_WIDTH = 147;
    public static final int A8_HEIGHT = 210;
    
    public static final String A9_NAME = "A9";
    public static final int A9_WIDTH = 105;
    public static final int A9_HEIGHT = 147;
    
    public static final String A10_NAME = "A10";
    public static final int A10_WIDTH = 74;
    public static final int A10_HEIGHT = 105;
    
    public static final String B0_NAME = "B0";
    public static final int B0_WIDTH = 2836;
    public static final int B0_HEIGHT = 4008;
    
    public static final String B1_NAME = "B1";
    public static final int B1_WIDTH = 2004;
    public static final int B1_HEIGHT = 2835;
    
    public static final String B2_NAME = "B2";
    public static final int B2_WIDTH = 1417;
    public static final int B2_HEIGHT = 2004;
    
    public static final String B3_NAME = "B3";
    public static final int B3_WIDTH = 1001;
    public static final int B3_HEIGHT = 1417;
    
    public static final String B4_NAME = "B4";
    public static final int B4_WIDTH = 709;
    public static final int B4_HEIGHT = 1001;
    
    public static final String B5_NAME = "B5";
    public static final int B5_WIDTH = 499;
    public static final int B5_HEIGHT = 709;
    
    public static final String B6_NAME = "B6";
    public static final int B6_WIDTH = 354;
    public static final int B6_HEIGHT = 499;
    
    public static final String B7_NAME = "B7";
    public static final int B7_WIDTH = 249;
    public static final int B7_HEIGHT = 354;
    
    public static final String B8_NAME = "B8";
    public static final int B8_WIDTH = 176;
    public static final int B8_HEIGHT = 249;
    
    public static final String B9_NAME = "B9";
    public static final int B9_WIDTH = 125;
    public static final int B9_HEIGHT = 176;
    
    public static final String B10_NAME = "B10";
    public static final int B10_WIDTH = 88;
    public static final int B10_HEIGHT = 125;
    
    public static final String LETTER_NAME = "LETTER";
    public static final int LETTER_WIDTH = 612;
    public static final int LETTER_HEIGHT = 792;
    
    public static final String TABLOID_NAME = "TABLOID";
    public static final int TABLOID_WIDTH = 792;
    public static final int TABLOID_HEIGHT = 1224;
    
    public static final String LEDGER_NAME = "LEDGER";
    public static final int LEDGER_WIDTH = 1224;
    public static final int LEDGER_HEIGHT = 792;
    
    public static final String LEGAL_NAME = "LEGAL";
    public static final int LEGAL_WIDTH = 612;
    public static final int LEGAL_HEIGHT = 1008;
    
    public static final String STATEMENT_NAME = "STATEMENT";
    public static final int STATEMENT_WIDTH = 396;
    public static final int STATEMENT_HEIGHT = 612;
    
    public static final String EXECUTIVE_NAME = "EXECUTIVE";
    public static final int EXECUTIVE_WIDTH = 540;
    public static final int EXECUTIVE_HEIGHT = 720;
    
    public static final String FOLIO_NAME = "FOLIO";
    public static final int FOLIO_WIDTH = 612;
    public static final int FOLIO_HEIGHT = 936;
    
    public static final String QUARTO_NAME = "QUARTO";
    public static final int QUARTO_WIDTH = 610;
    public static final int QUARTO_HEIGHT = 780;
    
    /**
     * A bunch of handy page formats defined statically
     */
    public static final PDFPageFormat A0 = new PDFPageFormat(A0_NAME, A0_HEIGHT, A0_WIDTH);
    
    public static final PDFPageFormat A1 = new PDFPageFormat(A1_NAME, A1_HEIGHT, A1_WIDTH);
    
    public static final PDFPageFormat A2 = new PDFPageFormat(A2_NAME, A2_HEIGHT, A2_WIDTH);
    
    public static final PDFPageFormat A3 = new PDFPageFormat(A3_NAME, A3_HEIGHT, A3_WIDTH);
    
    public static final PDFPageFormat A4 = new PDFPageFormat(A4_NAME, A4_HEIGHT, A4_WIDTH);
    
    public static final PDFPageFormat A5 = new PDFPageFormat(A5_NAME, A5_HEIGHT, A5_WIDTH);
    
    public static final PDFPageFormat A6 = new PDFPageFormat(A6_NAME, A6_HEIGHT, A6_WIDTH);
    
    public static final PDFPageFormat A7 = new PDFPageFormat(A7_NAME, A7_HEIGHT, A7_WIDTH);
    
    public static final PDFPageFormat A8 = new PDFPageFormat(A8_NAME, A8_HEIGHT, A8_WIDTH);
    
    public static final PDFPageFormat A9 = new PDFPageFormat(A9_NAME, A9_HEIGHT, A9_WIDTH);
    
    public static final PDFPageFormat A10 = new PDFPageFormat(A10_NAME, A10_HEIGHT, A10_WIDTH);
    
    public static final PDFPageFormat B0 = new PDFPageFormat(B0_NAME, B0_HEIGHT, B0_WIDTH);
    
    public static final PDFPageFormat B1 = new PDFPageFormat(B1_NAME, B1_HEIGHT, B1_WIDTH);
    
    public static final PDFPageFormat B2 = new PDFPageFormat(B2_NAME, B2_HEIGHT, B2_WIDTH);
    
    public static final PDFPageFormat B3 = new PDFPageFormat(B3_NAME, B3_HEIGHT, B3_WIDTH);
    
    public static final PDFPageFormat B4 = new PDFPageFormat(B4_NAME, B4_HEIGHT, B4_WIDTH);
    
    public static final PDFPageFormat B5 = new PDFPageFormat(B5_NAME, B5_HEIGHT, B5_WIDTH);
    
    public static final PDFPageFormat B6 = new PDFPageFormat(B6_NAME, B6_HEIGHT, B6_WIDTH);
    
    public static final PDFPageFormat B7 = new PDFPageFormat(B7_NAME, B7_HEIGHT, B7_WIDTH);
    
    public static final PDFPageFormat B8 = new PDFPageFormat(B8_NAME, B8_HEIGHT, B8_WIDTH);
    
    public static final PDFPageFormat B9 = new PDFPageFormat(B9_NAME, B9_HEIGHT, B9_WIDTH);
    
    public static final PDFPageFormat B10 = new PDFPageFormat(B10_NAME, B10_HEIGHT, B10_WIDTH);
    
    public static final PDFPageFormat LETTER = new PDFPageFormat(LETTER_NAME, LETTER_HEIGHT, LETTER_WIDTH);
    
    public static final PDFPageFormat TABLOID = new PDFPageFormat(TABLOID_NAME, TABLOID_HEIGHT, TABLOID_WIDTH);
    
    public static final PDFPageFormat LEDGER = new PDFPageFormat(LEDGER_NAME, LEDGER_HEIGHT, LEDGER_WIDTH);
    
    public static final PDFPageFormat LEGAL = new PDFPageFormat(LEGAL_NAME, LEGAL_HEIGHT, LEGAL_WIDTH);
    
    public static final PDFPageFormat STATEMENT = new PDFPageFormat(STATEMENT_NAME, STATEMENT_HEIGHT, STATEMENT_WIDTH);
    
    public static final PDFPageFormat EXECUTIVE = new PDFPageFormat(EXECUTIVE_NAME, EXECUTIVE_HEIGHT, EXECUTIVE_WIDTH);
    
    public static final PDFPageFormat FOLIO = new PDFPageFormat(FOLIO_NAME, FOLIO_HEIGHT, FOLIO_WIDTH);
    
    public static final PDFPageFormat QUARTO = new PDFPageFormat(QUARTO_NAME, QUARTO_HEIGHT, QUARTO_WIDTH);
    
    public static final Vector<PDFPageFormat> pageFormats = new Vector<PDFPageFormat>(Arrays.asList(new PDFPageFormat[] { A0, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, B0, B1, B2, B3, B4, B5, B6, B7, B8, B9, B10, LETTER, TABLOID, LEDGER, LEGAL, STATEMENT, EXECUTIVE, FOLIO, QUARTO }));
    
    /**
     * Constants for page orientation
     */
    public static final int LANDSCAPE = 0;
    public static final int PORTRAIT = 1;
    
    private String name;
    private int width;
    private int height;
    private int margin;
    private int orientation;
    
    
    
    public PDFPageFormat(String name, int height, int width, int margin, int orientation)
    {
        this.name = name;
        this.height = height;
        this.width = width;
        this.margin = margin;
        this.orientation = orientation;
    }
    
    
    
    public PDFPageFormat(String name, int height, int width)
    {
        this.name = name;
        this.height = height;
        this.width = width;
        this.margin = 36;
        this.orientation = PORTRAIT;
    }
    
    
    
    public PDFPageFormat()
    {
        this.name = LETTER_NAME;
        this.height = LETTER_HEIGHT;
        this.width = LETTER_WIDTH;
        this.margin = 36;
        this.orientation = PORTRAIT;
    }
    
    
    
    public PDFPageFormat(PDFPageFormat other)
    {
        this.name = other.name;
        this.height = other.height;
        this.width = other.width;
        this.margin = other.margin;
        this.orientation = other.orientation;
    }
    
    
    
    public String getName()
    {
        return name;
    }
    
    
    
    public int getWidth()
    {
        if (orientation == PORTRAIT)
        {
            return width;
        }
        else
        {
            return height;
        }
    }
    
    
    
    public int getHeight()
    {
        if (orientation == PORTRAIT)
        {
            return height;
        }
        else
        {
            return width;
        }
    }
    
    
    
    public int getMargin()
    {
        return margin;
    }
    
    
    
    public int getImageableWidth()
    {
        return getWidth() - 2 * margin;
    }
    
    
    
    public int getImageableHeight()
    {
        return getHeight() - 2 * margin;
    }
    
    
    
    public int getImageableX()
    {
        return margin;
    }
    
    
    
    public int getImageableY()
    {
        return margin;
    }
    
    
    
    public int getOrientation()
    {
        return orientation;
    }
    
    
    
    public void setName(String name)
    {
        this.name = name;
    }
    
    
    
    public void setWidth(int width)
    {
        if (orientation == PORTRAIT)
        {
            this.width = width;
        }
        else
        {
            this.height = width;
        }
    }
    
    
    
    public void setHeight(int height)
    {
        if (orientation == PORTRAIT)
        {
            this.height = height;
        }
        else
        {
            this.width = height;
        }
    }
    
    
    
    public void setMargin(int margin)
    {
        this.margin = margin;
    }
    
    
    
    public void setOrientation(int orientation)
    {
        this.orientation = orientation;
    }
    
    
    
    /**
     * Return the name so can build list for selection with ArrayAdapter
     */
    public String toString()
    {
        return name;
    }
    
    
    
    /**
     * Assess equality but ignoring orientation
     * 
     * @param other
     * @return
     */
    public boolean equals(PDFPageFormat other)
    {
        return ((this.name.equals(other.name)) && (this.width == other.width) && (this.height == other.height) && (this.margin == other.margin));
    }
    
}
