/*
 * GnuAPDF Library
 *   Jon Sevy, jsevy@cs.drexel.edu
 * 
 * Based on GnuJPDF Library, authored by (among others)
 *   Peter T Mount, http://www.retep.org.uk/pdf/
 *   Eric Z. Beard, ericzbeard@hotmail.com
 *   Gilbert DeLeeuw, gil1@users.sourceforge.net
 *   Matthew Hreljac, mhreljac@hotmail.com  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package gnu.apdf;


import java.io.IOException;
import java.io.OutputStream;

import android.graphics.Color;
import android.graphics.RectF;


/**
 * Base class for PDF shading types.
 * 
 * @author jsevy
 *
 */
public class PDFShading extends PDFStream
{
    
    /**
     * Required fields for any Shading
     */
    public int shadingType;
    public String colorSpace = "/RGB";
    
    // optional fields
    public Color backgroundColor = null;
    public RectF boundingBox = null;
    
    
    
    /**
     * Base class constructor for shading patterns; takes the shading type as argument:
     * <ul>
     * <li>1: Function-based shading</li>
     * <li>2: Axial shading</li>
     * <li>3: Radial shading</li>
     * <li>4: Free-form Gouraud-shaded triangle mesh</li>
     * <li>5: Lattice-form Gouraud-shaded triangle mesh</li>
     * <li>6: Coons patch mesh</li>
     * <li>7: Tensor-product patch mesh</li>
     * </ul>
     * 
     * @param shadingType
     */
    public PDFShading(int shadingType)
    {
        super("/Shading");
        this.shadingType = shadingType;
    }
    
    
    
    /**
     * SUperclass method for shading patterns; should be called first in subclass method. Writes the following to the
     * stream: /ShadingType <shadingType> /ColorSpace /RGB
     * 
     * @param os
     * @throws IOException
     */
    public void writeDictionary(OutputStream os) throws IOException
    {
        // write out the shading type
        os.write("/ShadingType ".getBytes());
        os.write(Integer.toString(shadingType).getBytes());
        os.write("\n".getBytes());
        
        // write out the color space
        os.write("/ColorSpace ".getBytes());
        os.write(colorSpace.getBytes());
        os.write("\n".getBytes());
        
    }
    
    
    
    public void write(OutputStream os) throws IOException
    {
        writeStart(os);
        
        // write out our remaining dictionary stuff
        writeDictionary(os);
        
        // write the contents of the Stream's byte buffer, along with the usual
        // header and footer
        writeStream(os);
        
        // Unlike most PDF objects, we don't call writeEnd(os) because we
        // extend a stream
    }
    
}