/*
 * GnuAPDF Library
 *   Jon Sevy, jsevy@cs.drexel.edu
 * 
 * Based on GnuJPDF Library, authored by (among others)
 *   Peter T Mount, http://www.retep.org.uk/pdf/
 *   Eric Z. Beard, ericzbeard@hotmail.com
 *   Gilbert DeLeeuw, gil1@users.sourceforge.net
 *   Matthew Hreljac, mhreljac@hotmail.com  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package gnu.apdf;


import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import android.graphics.Color;


/**
 * Class for the PDF Gouraud free-form shading object, used in implementing the drawVertices method in PDFCanvas.
 * 
 * @author jsevy
 *
 */
public class PDFShadingGouraudFreeform extends PDFShading
{
    
    /**
     * Required dictionary fields
     */
    private int bitsPerCoordinate = 32;
    private int bitsPerComponent = 8;
    private int bitsPerFlag = 8;
    private String decodeArray = new String();
    
    
    
    /**
     * Constructor for Gouraud free-form triangular mesh shading pattern, type 4. Used to implement drawVertices method
     * of PDFCanvas. The vertices array specifies the x,y pairs for each vertex. The colors array specifies the color at
     * each vertex; the colors at each vertex are interpolated across the corresponding triangle in a gradient.
     * <p>
     * The VertexMode determines how the subsequent points in the sequence of vertices are interpreted.
     * <ul>
     * <li>
     * TRIANGLES In this mode each triple of points (6 successive coordinates) is treated as a separate triangle. This
     * is the most general mode; no assumption is made about triangle adjacency.</li>
     * <li>
     * TRIANGLE_STRIP In this mode each successive vertex is combined with the two before it to form a new triangle,
     * giving a "strip" of triangles where each shares an edge with the triangle before it</li>
     * <li>
     * TRIANGLE_FAN In this mode the first point is assumed to be the center vertex for all of the triangles, with
     * subsequent points defining the "outer" vertices of the fan arranged radially around the center. The triangles are
     * thus "pie wedges" with the first point at the center of the pie, and each "wedge" sharing an edge and vertex.
     * Note that the first and last outer vertices are not automatically joined.</li>
     * </ul>
     * </p>
     * 
     * @param vertexMode
     *            Indicates how successive coordinate pairs are to be interpreted, as described above
     * @param vertexCount
     *            Number of vertices to include in the mesh
     * @param vertices
     *            Array of vertex coordinates, as a sequence of x,y pairs
     * @param vertexOffset
     *            Offset in vertex array of first coordinate to use for mesh
     * @param colors
     *            Array of colors specifying the color for each vertex
     * @param colorOffset
     *            Offset in color array for starting element
     */
    public PDFShadingGouraudFreeform(PDFCanvas.VertexMode vertexMode, int vertexCount, float[] vertices, int vertexOffset, int[] colors, int colorOffset)
    {
        // Gouraud free-form is shading type 4
        super(4);
        
        // determine the range of values needed for the vertex coordinates
        float vertexMax = 0;
        float vertexMin = 0;
        for (int i = 0; i < vertexCount; i++)
        {
            vertexMax = Math.max(vertexMax, vertices[i + vertexOffset]);
            vertexMin = Math.min(vertexMin, vertices[i + vertexOffset]);
        }
        
        // map vertex range into 0 to 2^32 - 1
        int[] scaledVertices = new int[vertexCount];
        for (int i = 0; i < vertexCount; i++)
        {
            scaledVertices[i] = (int) Math.floor(Integer.MAX_VALUE * (vertices[i + vertexOffset] - vertexMin) / (vertexMax - vertexMin));
            scaledVertices[i] = scaledVertices[i] << 1;
            // scaledVertices[i] = (int)vertices[i];
        }
        
        // decode array runs from the min to the max of the vertex components
        decodeArray = "[" + vertexMin + " " + vertexMax + " " + vertexMin + " " + vertexMax + " 0.0 1.0 0.0 1.0 0.0 1.0]";
        
        // generate the stream data into the base class byte array; will be
        // written out when the object is written out
        try
        {
            generateStreamData(vertexMode, vertexCount, scaledVertices, colors, colorOffset);
        }
        catch (IOException e)
        {
            // this should never happen, since we're really writing to a byte
            // array stream
            e.printStackTrace();
        }
    }
    
    
    
    /**
     * The stream data for the shaded triangular mesh consists of a sequence of vertex data structures f x y r g b where
     * x and y are the vertex coords in big-endian format and r, g and b are the RGB color components. f is a flag
     * indicating how triangles are to be formed to allow sharing vertices between adjacent triangles. If f = 0, there's
     * no sharing assumed bwteen the triangle currently being defined and any previous triangle; if f = 1, a triangle is
     * formed between this point (vertex n) and the two previous points (n-1 and n-2); and if f = 2 a triangle is formed
     * between this point (n), the previous point (n-1), and the one two before the previous (n-3).
     *
     * The vertex flags in the PDF vertex sequence are thus determined by the VertexMode, as follows:
     * <ul>
     * <li>TRIANGLES: use flag f = 0 for all vertices (no shared vertices in triangles)</li>
     * <li>TRIANGLE_STRIP: use flag f = 0 for first 3 vertices, then f = 1 for all subsequent</li>
     * <li>TRIANGLE_FAN: use flag f = 0 for first 3 vertices, then f = 2 for all subsequent</li>
     * </ul>
     * 
     * @param vertexMode
     * @param index
     * @return
     */
    private byte vertexFlag(PDFCanvas.VertexMode vertexMode, int index)
    {
        if (index < 3)
            return 0;
        else
        {
            switch (vertexMode)
            {
                case TRIANGLE_STRIP:
                    return 1;
                    
                case TRIANGLE_FAN:
                    return 2;
                    
                case TRIANGLES:
                default:
                    return 0;
                    
            }
        }
    }
    
    
    
    /**
     * The stream data for the shaded triangular mesh consists of a sequence of vertex data structures f x y r g b where
     * x and y are the vertex coordinates in big-endian format and r, g and b are the RGB color components. f is a flag
     * indicating how triangles are to be formed to allow sharing vertices between adjacent triangles. If f = 0, there's
     * no sharing assumed between the triangle currently being defined and any previous triangle; if f = 1, a triangle
     * is formed between this point (vertex n) and the two previous points (n-1 and n-2); and if f = 2 a triangle is
     * formed between this point (n), the previous point (n-1), and the one two before the previous (n-3).
     *
     * The vertex flags in the PDF vertex sequence are thus determined by the VertexMode, as follows:
     * <ul>
     * <li>TRIANGLES: use flag f = 0 for all vertices (no shared vertices in triangles)</li>
     * <li>TRIANGLE_STRIP: use flag f = 0 for first 3 vertices, then f = 1 for all subsequent</li>
     * <li>TRIANGLE_FAN: use flag f = 0 for first 3 vertices, then f = 2 for all subsequent</li>
     * </ul>
     * 
     * @param vertexMode
     * @param vertexCount
     * @param vertices
     * @param colors
     * @param colorOffset
     * @throws IOException
     */
    private void generateStreamData(PDFCanvas.VertexMode vertexMode, int vertexCount, int[] vertices, int[] colors, int colorOffset) throws IOException
    {
        // get our bytestream, and write our stream data
        DataOutputStream os = new DataOutputStream(getOutputStream());
        
        // write the vertex descriptor list
        for (int i = 0; i < vertexCount; i += 2)
        {
            os.writeByte(vertexFlag(vertexMode, i / 2));
            
            os.writeInt(vertices[i]);
            os.writeInt(vertices[i + 1]);
            
            os.writeByte(Color.red(colors[i / 2 + colorOffset]));
            os.writeByte(Color.green(colors[i / 2 + colorOffset]));
            os.writeByte(Color.blue(colors[i / 2 + colorOffset]));
        }
        
    }
    
    
    
    /**
     * Write relevant parameters to dictionary - here, BitsPerCoordinate, BitsPerComponent, BitsPerFlag, and the Decode
     * array.
     * 
     * @param os
     * @throws IOException
     */
    public void writeDictionary(OutputStream os) throws IOException
    {
        super.writeDictionary(os);
        
        // write out our fields
        os.write("/BitsPerCoordinate ".getBytes());
        os.write(Integer.toString(bitsPerCoordinate).getBytes());
        os.write("\n".getBytes());
        
        os.write("/BitsPerComponent ".getBytes());
        os.write(Integer.toString(bitsPerComponent).getBytes());
        os.write("\n".getBytes());
        
        os.write("/BitsPerFlag ".getBytes());
        os.write(Integer.toString(bitsPerFlag).getBytes());
        os.write("\n".getBytes());
        
        os.write("/Decode ".getBytes());
        os.write(decodeArray.getBytes());
        os.write("\n".getBytes());
        
    }
    
    
    
    /**
     * Method used for debugging - prints the information in the shading data stream.
     */
    public String getStreamDataString()
    {
        String dataString = new String();
        byte[] data = buf.toByteArray();
        
        for (int i = 0; i < data.length;)
        {
            // flag
            dataString += toHexString(data[i++]) + " ";
            
            // x coord
            dataString += toHexString(data[i++]) + toHexString(data[i++]) + toHexString(data[i++]) + toHexString(data[i++]) + " ";
            
            // y coord
            dataString += toHexString(data[i++]) + toHexString(data[i++]) + toHexString(data[i++]) + toHexString(data[i++]) + " ";
            
            // colors
            dataString += toHexString(data[i++]) + toHexString(data[i++]) + toHexString(data[i++]) + " ";
        }
        return dataString;
        
    }
    
}